from unittest import TestCase

from pychord import Chord

from pyharmonytools.displays.console_for_guitar_tab import ConsoleForGuitarTab
from pyharmonytools.displays.unit_test_report import UnitTestReport
from pyharmonytools.guitar.guitar_tab.guitar_tab import GuitarTab
from pyharmonytools.harmony.note import Note


def transform_expected_chords(expected_chords: object) -> object:
    """
    todo
    :param expected_chords:
    :return:
    """
    return None


class TestGuitarTab(TestCase):
    ut_report = UnitTestReport()

    def test_get_notes_from_chord_layout(self):
        chord_layout = [-1, -1, -1, 11, 11, 11]
        res = GuitarTab._get_notes_from_chord_layout(chord_layout)
        expected = [None, None, None, Note("Gb"), Note("Bb"), Note("Eb")]
        self.ut_report.assertTrue(res == expected)

    def test_digest_tab_dgbd(self):
        """ test_digest_tab_readme"""
        tab = """
        e|--11-----11-----10-----11----|
        B|--11-----12-----11-----11----|
        G|--11-----13-----10-----11----|
        D|-----------------------------|
        A|-----------------------------|
        E|-----------------------------|
        """
        #  2       9      16     23
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {"2": Chord("Ebm"), "9": Chord("G#m"), "16": Chord("Bb"), "23": Chord("Ebm")}
        gt = GuitarTab(tab)
        ConsoleForGuitarTab.display(gt)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_tab_gb6(self):
        tab = """
        e|--11---------------|
        B|------11-----------|
        G|-----------11------|
        D|-------------------|
        A|-------------------|
        E|-------------------|
        """
        expected = {"2": Chord("Bbsus/Gb")}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(expected, res))

    def test_digest_tab_gb6_d(self):
        tab = """
        e|--11------------2---|
        B|------11--------3---|
        G|-----------11---2---|
        D|--------------------|
        A|--------------------|
        E|--------------------|
        """
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {"2": Chord("Ebm"), "16": Chord("D")}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_tab_eb(self):
        tab = """
        e|--11----|
        B|--------|
        G|--------|
        D|--------|
        A|--------|
        E|--------|
        """
        expected = {"2": Chord("Eb")}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_tab_full_bach(self):
        # todo work on expected chord + digesting full tab
        #  (multiple bars & lines with tab signs such as hammering/pull off/...)
        # see https://tabs.ultimate-guitar.com/tab/johann-sebastian-bach/cello-suite-no-1-prelude-tabs-799617
        #             Cello Suite No. 1 - Prelude
        #             Johann Sebastian Bach
        #             4/4
        #             Key: G Major
        tab_full = """
            e|---------------------------------|---------------------------------|
            B|-----0---0---0-------0---0---0---|-----1-0-1---1-------1-0-1---1---|
            G|-------2---------------2---------|---------------------------------|
            D|---0-------0---0---0-------0---0-|---2-------2---2---2-------2---2-|
            A|---------------------------------|---------------------------------|
            E|-3---------------3---------------|-3---------------3---------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|-----0---0---0-------0---0---0---|
            G|-----5-4-5---5-------5-4-5---5---|---0---2---0---0---0---2---0-----|
            D|---4-------4---4---4-------4---4-|-------------------------------4-|
            A|---------------------------------|---------------------------------|
            E|-3---------------3---------------|-3---------------3---------------|
             
            e|---------------------------------|---------------------------------|
            B|-----0---0-----------------------|---------------------------------|
            G|-------2---0---0---0---0---------|---0-2-0-2-0-2-0---0-2-0-2-0-2-0-|
            D|---2---------4---2---4-----0-----|---------------------------------|
            A|-------------------------2---4-2-|-4---------------4---------------|
            E|-3-------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|-----3-2-3-----------------------|---------------------------------|
            G|---2-------2-0-2---2-0-2---------|-----0---0---0-------0---0---0---|
            D|-4---------------4-------0-4-2-0-|-------4---------------4---------|
            A|---------------------------------|---2-------2---2---2-------2---2-|
            E|---------------------------------|-0---------------0---------------|
             
            e|---------------------------------|---------------------------------|
            B|-----------------------3-2-0-----|-------3---3---------------------|
            G|-----------------0-----------2-0-|---------2-----2-------2-0-------|
            D|-----0-2-0---------4-2-----------|-4-2-0-------4---0-2-4-----4-2-0-|
            A|---4-------4-2-0-----------------|---------------------------------|
            E|-0-------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|-------0-1-------------0-1-------|
            G|-1-----------1---4-----------1---|-----2-----2---------2-----2-----|
            D|---0-3-2-3-0---0---0-3-2-3-0---0-|---2---------2-0---2---------4-2-|
            A|---------------------------------|-3---------------3---------------|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|---------------------------------|
            G|---------------------------------|-0-----0---0-2---0---------------|
            D|---4---4-7-4-7-4---4---4-7-4-7-4-|---4-2---4-----4---4-2-0---------|
            A|-6---6-----------6---6-----------|-------------------------3-2-0---|
            E|---------------------------------|-------------------------------3-|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|---------------------------------|
            G|---------------------------------|---------------------------------|
            D|-----0---0---0-------0---0---0---|-----3-2-3---3-------3-2-3---3---|
            A|---3---3---3---3---3---3---3---3-|---2-------2---2---2-------2---2-|
            E|-2---------------2---------------|-3---------------3---------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|-------0---------------0---------|
            G|---------------------------------|-----5---5---5-------5---5---5---|
            D|-----2-0-2---2-------2-0-2---2---|---4-------4---4---4-------4---4-|
            A|---3-------3---3---3-------3---3-|---------------------------------|
            E|-3---------------3---------------|-3---------------3---------------|
             
            e|---------------------------------|---------------------------------|
            B|-----0---0-----------------------|---------------------------------|
            G|-------2---0---------------------|---------0-----0---------0-----0-|
            D|---0---------4-2-0-------------0-|-----2-4---2-4-------2-4---2-4---|
            A|-------------------3-2-0---------|-4-0-------------3-0-------------|
            E|-3-----------------------3-2-0---|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|-----------2-3-------------------|
            G|---------------------------------|---------2---------------------0-|
            D|-----0-2-4-0-2-4-----0-2-4-0-2-4-|-----0-4-----------------0-2-4---|
            A|-3-0-------------3-0-------------|-3-0---------------0-2-3---------|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------0-1---------0-1-3-|-4-3-2-3-3-1-0-1-1---------------|
            G|-2---------0-2-----2---0-2-------|-------------------2-------------|
            D|---4-0-2-4-----------4-----------|---------------------4-2-0-------|
            A|---------------------------------|---------------------------0-2-3-|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|-----------0-1---0---------------|-------------0---2---------------|
            G|---------2-----2---0-------------|---------0-2---0---3-2-3-3-2-1-2-|
            D|-0---0-4-------------0-----------|-0-----0-------------------------|
            A|---0-------------------3-2---0-2-|-----2---------------------------|
            E|---------------------------3-----|---3-----------------------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------2-3-2-|-3-------------------------------|
            G|-2-0---0-0-------------0-2-------|---2-------2---------------------|
            D|-----4-----2---------2-----------|-----4-2-4---0-4---0-------------|
            A|-------------4-2-0-4-------------|-----------------0---4-2-0-------|
            E|---------------------------------|---------------------------3-2-0-|
             
            e|---------------------------------|---------------------------------|
            B|-----1-0-----------1-0-----------|---0-----------------------------|
            G|---------2-0-----------2-0-------|-----2-0-----------2-0-----------|
            D|-0-----------4-2-0---------4-2-0-|---------4-2-0---------4-2-0-----|
            A|---------------------------------|-3-------------3-2-----------3-2-|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|---------------------------------|---------------------------------|
            G|---0-------2---2---2---2-0-2---2-|---2---2-0-2---2---2---2-0-2---2-|
            D|-----4-2-4---0---2---4-------2---|-4---0-------2---4---0-------2---|
            A|-0-------------------------------|---------------------------------|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|-------------------------0-------|-----0---1-------0---1---3---0---|
            G|---2---2---2---2-0-2-2-2---2---2-|-2-2---2---2---2---2---2---2---2-|
            D|-4---0---2---4---------------0---|-------------0-------------------|
            A|---------------------------------|---------------------------------|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------------------|
            B|-1---0---1-------0-------0-------|---------------------------------|
            G|---2---2---2-2-2---2-2-2---2-0-2-|-2-2-0-2-2-2---2-0-2---2-0-2---2-|
            D|---------------------------------|-------------4-------4-------2---|
            A|---------------------------------|---------------------------------|
            E|---------------------------------|---------------------------------|
             
            e|---------------------------------|---------------------0---1---2---|
            B|---------------------------------|-0---1---2---3---4---------------|
            G|---2-------------0---1---2---3---|---------------------------------|
            D|-4---0-2-3-0-4-0---0---0---0---0-|---0---0---0---0---0---0---0---0-|
            A|---------------------------------|---------------------------------|
            E|---------------------------------|---------------------------------|
             
            e|-3-------3---3---3-------3---3---|-3-------3---3---3-------3---3---|
            B|---0---0---0---0---0---0---0---0-|---------------------------------|
            G|---------------------------------|---2---2---2---2---2---2---2---2-|
            D|-----0---------------0-----------|-----0---------------0-----------|
            A|---------------------------------|---------------------------------|
            E|---------------------------------|---------------------------------|
             
            e|-2-------2---2---2-------2---2---|-3-------------------------------|
            B|---1---1---1---1---1---1---1---1-|-0-------------------------------|
            G|---------------------------------|---------------------------------|
            D|-----0---------------0-----------|---------------------------------|
            A|---------------------------------|---------------------------------|
            E|---------------------------------|-3-------------------------------|        
        """
        # not yet reliable
        expected_chords = """
        C     C     Dm  Dm7 G7  G7  C   C
        C7    C7    D7  D7  G   G   C   C
        Am    Am    D7  D7  G   G   Gdim  Edim
        Dm    Dm    Ddim  Fdim  C   C   F   F
        Dm    Dm7   G7  G7  C   C   C7  Fmaj7
        Fmaj7 Fmaj7 F#dim F#dim C#dim Fdim  G7  G7
        C     C     G7  G7  G7  G7  Cm  C
        C     C     G   G   G7  G7  C7  C7
        C     F     C   C   C   C
        """
        gt = GuitarTab(tab_full)
        ConsoleForGuitarTab.display(gt)
        # the chord transcription will be too hard to test => simple check on processed bars qty
        self.ut_report.assertTrue(gt.get_number_of_bars() == 42)

    def test_digest_bach_bar_1(self):
        tab = """
          e|---------------------------------|
          B|-----0---0---0-------0---0---0---|
          G|-------2---------------2---------|
          D|---0-------0---0---0-------0---0-|
          A|---------------------------------|
          E|-3---------------3---------------|
        """
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {'1': Chord("Gadd9")}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_tab_simplest_progressive_chords_in_a_bar_ebm(self):
        tab = """
        e|--11---------------|
        B|------11-----------|
        G|-----------11------|
        D|-------------------|
        A|-------------------|
        E|-------------------|
        """
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {"2": Chord("Ebm")}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_bach_bar_1_2(self):
        # the involved method is based on building chords
        # as long as the fret range is acceptable (Fingering.FINGERING_WIDTH)
        # and you have enough fingers or a tab
        # therefore, sometimes the chord guessing is not inaccurate
        # since musical phrases are not taken into account
        tab = """
            e|-------------------------------------------------------------------|
            B|-----0---0---0-------0---0---0---------1-0-1---1-------1-0-1---1---|
            G|-------2---------------2-------------------------------------------|
            D|---0-------0---0---0-------0---0-----2-------2---2---2-------2---2-|
            A|-------------------------------------------------------------------|
            E|-3---------------3-----------------3---------------3---------------|
            """
        #      1                                 35    4143            5759
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {'1': Chord('Gadd9'), '37': Chord('C/G')}    # the split of the chord does not match bars
                                                                # (originally at caret 35)
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_bach_bar_1_2_with_bar(self):
        # demonstrates the note in UT "test_digest_bach_bar_1_2"
        tab = """
            e|---------------------------------|---------------------------------|
            B|-----0---0---0-------0---0---0---|-----1-0-1---1-------1-0-1---1---|
            G|-------2---------------2---------|---------------------------------|
            D|---0-------0---0---0-------0---0-|---2-------2---2---2-------2---2-|
            A|---------------------------------|---------------------------------|
            E|-3---------------3---------------|-3---------------3---------------|
        """
        #      1                                 35    4143            5759
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {'1': Chord('Gadd9')}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))
        expected = {'1': Chord('C/G')}
        res = gt.get_simplest_progressive_chords_in_a_bar(1)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def test_digest_tab_simplest_progressive_chords_in_a_bar_gb6_d(self):
        tab = """
        e|--11------------2---|
        B|------11--------3---|
        G|-----------11---2---|
        D|--------------------|
        A|--------------------|
        E|--------------------|
        """
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {"2": Chord("Ebm"), "16": Chord("D")}
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def _test_digest_tab_simplest_progressive_chords_with_notations_1_3(self):
        # https://tabs.ultimate-guitar.com/tab/eagles/hotel-california-tabs-94065
        tab = r"""
            e|-------------------------------|------------------------|------------------------------------|
            B|--(15)--10-12-------------7----|------------------------|------7-------------7---------------|
            G|--------------11s9~~--7h9---7^-|------------------------|--7h9---7^-------7h9---7^-----------|
            D|-------------------------------|-8---9h12p9--7--(8)---\-|----------9p8-9------------9p7-9p7--|
            A|-------------------------------|------------------------|------------------------------------|
            E|-------------------------------|------------------------|------------------------------------|
            """
        gt = GuitarTab(tab)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        ConsoleForGuitarTab.display(gt)
        # Bm is provided on https://tabs.ultimate-guitar.com/tab/eagles/hotel-california-tabs-94065
        # Bm notes: B, F# D
        # played notes: (D), A, B, F# s _E_  , D, E, F#, D
        # but the heard notes are (A), B, D, E, (F#)
        # so the chord should sound like E9sus4 or Bm7/E or D6/E
        # see https://www.oolimo.com/guitarchords/analyze
        expected = {"2": Chord("D6/E")}
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))
        res = gt.get_simplest_progressive_chords_in_a_bar(1)
        expected = {"1": Chord("F#7")}
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))
        res = gt.get_simplest_progressive_chords_in_a_bar(2)
        expected = {"2": Chord("A")}
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))

    def _test_digest_tab_simplest_progressive_chords_with_notations(self):
        # https://tabs.ultimate-guitar.com/tab/eagles/hotel-california-tabs-94065
        tab = """
  Bm               ~~        1/4  F#7                      A       1/2           1/2
e|-------------------------------|------------------------|------------------------------------|
B|--(15)--10-12-------------7----|------------------------|------7-------------7---------------|
G|--------------11s9~~--7h9---7^-|------------------------|--7h9---7^-------7h9---7^-----------|
D|-------------------------------|-8---9h12p9--7--(8)---\-|----------9p8-9------------9p7-9p7--|
A|-------------------------------|------------------------|------------------------------------|
E|-------------------------------|------------------------|------------------------------------|

  E                                           G                                  D
e|--------10--------------------7---10^(12)-|--10-(12)10p7---------10----------|--------10--------------|
B|10^(12)----(12)10p7-10-8^(9)----7---------|--------------10^(12)----(12)10-8-|------------------------|
G|------------------------------------------|----------------------------------|-9^(11)----(11)9p7-7p6-7|
D|------------------------------------------|----------------------------------|------------------------|
A|------------------------------------------|----------------------------------|------------------------|
E|------------------------------------------|----------------------------------|------------------------|

  Em                                                   F#7                                    ~~
e|----------------------------------------------------|----------------------7--------------------|
B|------7^(8)-7^(8)(7)-------7^(8)-7^(8)-7^(8)(7)-----|--------7-10--------7----------------------|
G|--7h9----------------9p7-9----------------------9p7-|-9^(11)------9^(11)-----9^(11)9p7-9^(11)~~-|
D|----------------------------------------------------|-------------------------------------------|
A|----------------------------------------------------|-------------------------------------------|
E|----------------------------------------------------|-------------------------------------------|

[Second solo part]
  Bm                                                                F#7    ~~~
e|-----------------------7----------7--------------7----------------|-------------|
B|--------7--10--------7----------7-------------7-------------------|-------------|
G|-9^(11)-------9^(11)-----9^(11)--------9^(11)------9^(11)-9p7-9-7-|--9^(11)~~~--|
D|------------------------------------------------------------------|-------------|
A|------------------------------------------------------------------|-------------|
E|------------------------------------------------------------------|-------------|

  A                   1/4                        E9                 ~~
e|-------------------------------------7--------|-------10------------------|
B|-----------------------------10^(12)-----10-7-|-10s12----12p11-10-9~~/----|
G|----11^(14)(11)-9-11-7^-----------------------|------------------------14-|
D|---x---------------------9--------------------|---------------------------|
A|--x-------------------------------------------|---------------------------|
E|-x--------------------------------------------|---------------------------|

  G                                                    D                                    ~~
e|-------------------------17--------------------------|-15-16-17-17^(19)----------------------|
B|--------12-15-12-15^(17)----17--15--17--15--17^(19)--|-----------------(19)(17)-15~~-17-17p15|
G|^(16)-16---------------------------------------------|---------------------------------------|
D|-----------------------------------------------------|---------------------------------------|
A|-----------------------------------------------------|---------------------------------------|
E|-----------------------------------------------------|---------------------------------------| 

  Em                           ~~                 F#7
e|-----------17--17------------------------------|-----------17--17------------------------------|
B|--17^(19)---------(19)(18)(19)~~--(19)(17)--15-|--17^(19)---------(19)(18)(19)~~--(19)(17)--15-|
G|-----------------------------------------------|-----------------------------------------------|
D|-----------------------------------------------|-----------------------------------------------|
A|-----------------------------------------------|-----------------------------------------------|
E|-----------------------------------------------|-----------------------------------------------|
 
[Third solo part]
  Bm                                                F#7
e|--17^(19)-------------10---------10--------------|-----------------|
B|--------------12^(15)----12^(15)-----------------|--11~~~~~~~~~~~~-|
G|-------------------------------------11s9p7------|-----------------|
D|--------------------------------------------9--8-|-----------------|
A|-------------------------------------------------|-----------------|
E|-------------------------------------------------|-----------------|

  A             Bend and gradually release                   1/4  E9                        ~~
e|-----12^(14)-(13)(13)(13)(13)(13)(13)-12-12-12-12p10----12-10^-|----------------------------------------|
B|---x-------------------------------------------------12--------|--12~~--10---------------10~~-\---------|
G|--x------------------------------------------------------------|-----------11s9--7--9s11---------9^(11)-|
D|---------------------------------------------------------------|----------------------------------------|
A|---------------------------------------------------------------|----------------------------------------|
E|---------------------------------------------------------------|----------------------------------------|
 
  G                                      D                         ~~~
e|--10----9---7-------------------------|---14--12-10-----------------------|
B|-----10---8---7-10---8----------------|-----------------------------------|
G|-------------------7-----7------------|-7-----------12^(14)(12)-11~~~--11-|
D|-----------------------9---7s5p4-5s7--|-----------------------------------|
A|--------------------------------------|-----------------------------------|
E|--------------------------------------|-----------------------------------|

  Em                              F#7
e|-------------------------------|------------9------10------12-|
B|------------------10------12---|---11-------------------------|
G|--12------12-------------------|------------------------------|
D|-------------------------------|------------------------------|
A|-------------------------------|------------------------------|
E|-------------------------------|------------------------------|

[Outro Part] (Play twice)
  Bm                                        F#7                         A
e|14p10---14p10---14p10---14p10---14p10---|-12p9---12p9---12p9-9s18~~|-12p9---12p9---12p9---12p9---12p9--|
B|-----12------12------12------12------12-|-----11-----11------------|-----10-----10-----10-----10-----10|
G|----------------------------------------|--------------------------|-----------------------------------|
D|----------------------------------------|--------------------------|-----------------------------------|
A|----------------------------------------|--------------------------|-----------------------------------|
E|----------------------------------------|--------------------------|-----------------------------------|

  E9                      G                               D
e|-10p7--10p7--10p7-7s16~~-|-10p7--10p7--10p7--10p7--10p7--|-10-----10-----10---10s14~~-|
B|-----9-----9-------------|-----8-----8-----8-----8-----8-|---10-----10-----10---------|
G|-------------------------|-------------------------------|-----11-----11--------------|
D|-------------------------|-------------------------------|----------------------------|

A|-------------------------|-------------------------------|----------------------------|
E|-------------------------|-------------------------------|----------------------------|

  Em
e|-7---7---7---7---7---|-9-----9----9---9s14~~-|
B|--8---8---8---8---8--|--11----11---11--------|
G|---9---9---9---9---9-|----11----11-----------|
D|---------------------|-----------------------| Repeat Outro part, then play:
A|---------------------|-----------------------|
E|---------------------|-----------------------|

 

  B
e|----------|
B|----------|
G|----------|
D|-9-9-9-9--|
A|-9-9-9-9--|
E|-7-7-7-7--|
        """
        # ************************************
        # | h   Hammer-on
        # | p   Pull-off
        # | ^(n)Bend
        # | /   Slide up
        # | \   Slide down
        # | ~   Vibrato
        # ************************************
        # checked with https://www.oolimo.com/guitarchords/analyze
        expected = {"2": Chord("Ebm"), "16": Chord("D")}
        gt = GuitarTab(tab)
        ConsoleForGuitarTab.display(gt)
        res = gt.get_simplest_progressive_chords_in_a_bar(0)
        self.ut_report.assertTrue(GuitarTab.are_same_digested_tabs(res, expected))
